# src/lightwave/config/business/__init__.py
__version__ = "0.0.1"

# src/lightwave/config/business/__init__.py
# Empty file is fine

# src/lightwave/config/business/__init__.py
from .principles import PrinciplesConfig, BusinessPrinciple, PrincipleCategory

__all__ = ['PrinciplesConfig', 'BusinessPrinciple', 'PrincipleCategory']